{
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::    Author: Philip Bairstow.                                                ::
::    E-Mail: Philip@pbairstow.com                                            ::
::    www: www.pbairstow.com                                                  ::
::    ICQ: 47850165                                                           ::
::                                                                            ::
::    04 June 2000.                                                           ::
::                                                                            ::
::    Version information:                                                    ::
::                                                                            ::
::    Version 1 - Gives user the ability to Read/Write String, Integer and    ::
::                Boolean values entries TO/FROM the Registry.                ::
::                Value is Stored in the Following Way:                       ::
::                                                                            ::
::                RegistryBaseKey\CompanyName\ProgramName\ProgramVersion.     ::
::                                                                            ::
::                                                                            ::
::    If you use this product then you agree with the Following:              ::
::    THAT THE AUTHOR DOES NOT ACCEPT RESPONSABILITY FOR THE LOSS OR DAMAGE   ::
::    CAUSED EITHER DIRECTLY OR INDIRECTLY BY USING THIS SOFTWARE.            ::
::    YOU MAY NOT SELL OR PROFIT IN ANY WAY FROM THIS SOFTWARE WITHOUT THE    ::
::    WRITTEN PERMISSION FROM THE AUTHOR AND ANY ASSOCIATES.                  ::
::    THIS SOFTWARE MAY BE FREELY DISTRIBUTABLE AS ALONG AS ALL ORIGIONAL     ::
::    FILES ARE UNCHANGED AND PRESENT. YOU MUST REPORT ANY BUGS TO THE EMAIL  ::
::    ADDRESS IN THE README.TXT FILE.                                         ::
::                                                                            ::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
}

unit EasyRegistry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry;

type
  TRegBaseKey = (bkHKEY_CLASSES_ROOT, bkHKEY_CURRENT_USER, bkHKEY_LOCAL_MACHINE, bkHKEY_USERS, bkHKEY_CURRENT_CONFIG);
  TEasyRegistry = class(TComponent)
  private
    FReg: TRegistry;
  protected
    FCompanyName,
    FProgramName,
    FProgramVersion,
    FBaseKey: String;
    FRegBaseKey: TRegBaseKey;
    FCryptKey: String;
    Crypt_Key: Integer;
    {Returns a HKEY Equivelent to the Value Of FRegBaseKeystry}
    Function SetBaseKey(Key: TRegBaseKey):HKEY;
    {Encryts String Values}
    Function Crypt(S: String; CRYPT_Key: Integer): string;
  public
    {Writes a String Value to the Registry}
    Procedure WriteString(Subkey,Value: String);
    {Returns a String Value From the Registry}
    Function ReadString(Subkey: String): String;
    {Writes a Integer to the Registry}
    Procedure WriteInteger(Subkey: String; Value: integer);
    {Reads a Integer From the Registry}
    Function ReadInteger(Subkey: String):Integer;
    {Writes a Boolean Value to the Registry}
    Procedure WriteBoolean(Subkey: String; Value: Boolean);
    {Reads a Boolean Value From the Registry}
    Function ReadBoolean(Subkey: String): Boolean;
  published
    {Sets the Registry Base Key}
    Property RegistryBaseKey: TRegBaseKey read FRegBaseKey write FRegBaseKey;
    {Sets the Value used in Encyption}
    Property SetCrypt_Key: Integer read Crypt_Key write Crypt_key Default 50;
    {Sets the Company Name}
    Property CompanyName: String read FCompanyName write FCompanyName;
    {Sets the Program Version}
    Property ProgramVersion: String read FProgramVersion write FProgramVersion;
    {Sets the Program Name}
    Property ProgramName: String read FProgramName write FProgramName;
  end;

procedure Register;
var
  Temp: String;
  FRegistryBaseKey: HKEY;
  
implementation

procedure Register;
begin
  RegisterComponents('PBSoftware', [TEasyRegistry]);
end;
                      
Function TEasyRegistry.SetBaseKey(Key: TRegBaseKey):HKEY;
begin
  If Key = bkHKEY_CLASSES_ROOT then Result := HKEY_CLASSES_ROOT else
  If Key = bkHKEY_CURRENT_USER then Result := HKEY_CURRENT_USER else
  If Key = bkHKEY_LOCAL_MACHINE then Result := HKEY_LOCAL_MACHINE else
  If Key = bkHKEY_USERS then Result := HKEY_USERS else
  If Key = bkHKEY_CURRENT_CONFIG then Result := HKEY_CURRENT_CONFIG else
  ERegistryException.Create('Invalid Key Type');
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.Crypt(S: String; CRYPT_Key: Integer): string;
var
  tmpstr: string;
  x: integer;
  b: byte;
begin
  tmpstr:= '';
  for x:= 1 to length(S) do
  begin
    b:= byte(S[x]) xor CRYPT_KEY;
    tmpstr:= tmpstr + char(b);
  end;
  result:= tmpstr;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteString(Subkey, Value: String);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    begin
      FCryptKey := Crypt(Value, Crypt_Key);
      FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
      FReg := TRegistry.Create;
      Try
         FReg.RootKey := FRegistryBaseKey;
         FReg.CreateKey(FBaseKey);
         FReg.OpenKey(FBaseKey, True);
         FReg.WriteString(Subkey, FCryptKey);
      Finally
         FReg.Free;
      end
  end;

  Except
      On E:ERegistryException do Raise;
      end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadString(Subkey: String): String;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
    Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBaseKey);
       FReg.OpenKey(FBaseKey, True);
       Temp := FReg.ReadString(Subkey);
       Result := Crypt(Temp, Crypt_Key);
    Finally
       FReg.Free;
    end;

  Except
      On E:ERegistryException do Raise;
      end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteInteger(Subkey: String; Value: Integer);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
      Try
        FReg.Rootkey := FRegistryBaseKey;
        FReg.CreateKey(FBasekey);
        FReg.OpenKey(FBaseKey, True);
        FReg.WriteInteger(SubKey, Value);
      Finally
        FReg.Free;
        end;
  Except
      On E:ERegistryException do Raise;
      end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadInteger(SubKey: String): Integer;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
     FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
     FReg := TRegistry.Create;
     Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBasekey);
       FReg.OpenKey(FBaseKey, True);
       Result := FReg.ReadInteger(Subkey);
     Finally
       FReg.Free;
       end;

  Except
      On E:ERegistryException do Raise;
      end;
end;

//------------------------------------------------------------------------------

Procedure TEasyRegistry.WriteBoolean(SubKey: String; Value: Boolean);
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
     Try
       FReg.Rootkey := FRegistryBaseKey;
       FReg.CreateKey(FBasekey);
       FReg.OpenKey(FBaseKey, True);
       FReg.WriteBool(Subkey, Value);
     Finally
       FReg.Free;
     end;

  Except
      On E:ERegistryException do Raise;
      end;
end;

//------------------------------------------------------------------------------

Function TEasyRegistry.ReadBoolean(Subkey: String): Boolean;
begin
  FRegistryBaseKey := SetBaseKey(FRegBaseKey);
  Try
    FBasekey := ConCat('Software','\',CompanyName,'\',ProgramName,'\',ProgramVersion,'\');
    FReg := TRegistry.Create;
    Try
      FReg.Rootkey := FRegistryBaseKey;
      FReg.CreateKey(FBaseKey);
      FReg.OpenKey(FBaseKey, True);
      Result := FReg.ReadBool(Subkey);
    Finally
      FReg.Free;
    end;

  Except
      On E:ERegistryException do Raise;
      end;
end;

end.
